// pev_changelog by VEN

#include <amxmodx>
#include <engine>

// comment/uncomment to disable/enable int/fl/vec pev change logging
#define LOG_INT
#define LOG_FL
#define LOG_VEC

// target entity index
stock const id = 1

// exclude pev list
stock const g_IntPevExcluded[] = {-1}
stock const g_FlPevExcluded[] = {-1}
stock const g_VecPevExcluded[] = {-1}

// that's what i've exluded
//stock const g_IntPevExcluded[] = {1, 10, 12, 13, 18, 19, 21, 27, 34}
//stock const g_FlPevExcluded[] = {10, 11, 25}
//stock const g_VecPevExcluded[] = {0, 2, 6, 9, 12, 13}

stock const g_IntPevName[][] = {
	"INT_gamestate",
	"INT_oldbuttons",
	"INT_groupinfo",
	"INT_iuser1",
	"INT_iuser2",
	"INT_iuser3",
	"INT_iuser4",
	"INT_weaponanim",
	"INT_pushmsec",
	"INT_bInDuck",
	"INT_flTimeStepSound",
	"INT_flSwimTime",
	"INT_flDuckTime",
	"INT_iStepLeft",
	"INT_movetype",
	"INT_solid",
	"INT_skin",
	"INT_body",
	"INT_effects",
	"INT_light_level",
	"INT_sequence",
	"INT_gaitsequence",
	"INT_modelindex",
	"INT_playerclass",
	"INT_waterlevel",
	"INT_watertype",
	"INT_spawnflags",
	"INT_flags",
	"INT_colormap",
	"INT_team",
	"INT_fixangle",
	"INT_weapons",
	"INT_rendermode",
	"INT_renderfx",
	"INT_button",
	"INT_impulse",
	"INT_deadflag"
}

stock const g_FlPevName[][] = {
	"FL_impacttime",
	"FL_starttime",
	"FL_idealpitch",
	"FL_pitch_speed",
	"FL_ideal_yaw",
	"FL_yaw_speed",
	"FL_ltime",
	"FL_nextthink",
	"FL_gravity",
	"FL_friction",
	"FL_frame",
	"FL_animtime",
	"FL_framerate",
	"FL_health",
	"FL_frags",
	"FL_takedamage",
	"FL_max_health",
	"FL_teleport_time",
	"FL_armortype",
	"FL_armorvalue",
	"FL_dmg_take",
	"FL_dmg_save",
	"FL_dmg",
	"FL_dmgtime",
	"FL_speed",
	"FL_air_finished",
	"FL_pain_finished",
	"FL_radsuit_finished",
	"FL_scale",
	"FL_renderamt",
	"FL_maxspeed",
	"FL_fov",
	"FL_flFallVelocity",
	"FL_fuser1",
	"FL_fuser2",
	"FL_fuser3",
	"FL_fuser4"
}

stock const g_VecPevName[][] = {
	"VEC_origin",
	"VEC_oldorigin",
	"VEC_velocity",
	"VEC_basevelocity",
	"VEC_clbasevelocity",
	"VEC_movedir",
	"VEC_angles",
	"VEC_avelocity",
	"VEC_punchangle",
	"VEC_v_angle",
	"VEC_endpos",
	"VEC_startpos",
	"VEC_absmin",
	"VEC_absmax",
	"VEC_mins",
	"VEC_maxs",
	"VEC_size",
	"VEC_rendercolor",
	"VEC_view_ofs",
	"VEC_vuser1",
	"VEC_vuser2",
	"VEC_vuser3",
	"VEC_vuser4"
}

public server_frame() {
	if (!is_valid_ent(id))
		return

	static i, Float:gametime
	gametime = get_gametime()

#if defined LOG_INT
	static IntPevTempVal, IntPevVal[sizeof g_IntPevName]
	for (i = 0; i < sizeof g_IntPevName; i++) {
		IntPevTempVal = entity_get_int(id, i)
		if (IntPevTempVal != IntPevVal[i] && !IsPevExcluded(i, g_IntPevExcluded, sizeof g_IntPevExcluded)) {
			server_print("%2d  %-19s   %11d   %11d   %.10f", i, g_IntPevName[i], IntPevVal[i], IntPevTempVal, gametime)
			IntPevVal[i] = IntPevTempVal
		}
	}
#endif

#if defined LOG_FL
	static Float:FlPevTempVal, Float:FlPevVal[sizeof g_FlPevName]
	for (i = 0; i < sizeof g_FlPevName; i++) {
		FlPevTempVal = entity_get_float(id, i)
		if (FlPevTempVal != FlPevVal[i] && !IsPevExcluded(i, g_FlPevExcluded, sizeof g_FlPevExcluded)) {
			server_print("%2d  %-19s   %10.3f   %10.3f   %.10f", i, g_FlPevName[i], FlPevVal[i], FlPevTempVal, gametime)
			FlPevVal[i] = FlPevTempVal
		}
	}
#endif

#if defined LOG_VEC
	static Float:VecPevTempVal[3], Float:VecPevVal[sizeof g_VecPevName][3]
	for (i = 0; i < sizeof g_VecPevName; i++) {
		entity_get_vector(id, i, VecPevTempVal)
		if (!IsVecEqual(VecPevTempVal, VecPevVal[i]) && !IsPevExcluded(i, g_VecPevExcluded, sizeof g_VecPevExcluded)) {
			server_print("%2d  %-18s   %10.3f   %10.3f   %.10f", i, g_VecPevName[i], VecPevVal[i][0], VecPevTempVal[0], gametime)
			server_print("                         %10.3f   %10.3f", VecPevVal[i][1], VecPevTempVal[1])
			server_print("                         %10.3f   %10.3f", VecPevVal[i][2], VecPevTempVal[2])
			server_print("-------------------------------------------------------------------------")
			VecPevVal[i] = VecPevTempVal
		}
	}
#endif
}

stock bool:IsPevExcluded(PevIndex, const PevExcluded[], PevExcludedTotalNum) {
	static i
	for (i = 0; i < PevExcludedTotalNum; i++)
		if (PevExcluded[i] == PevIndex)
			return true

	return false
}

stock bool:IsVecEqual(const Float:vec1[], const Float:vec2[]) { // stolen from XS (laziness)
	return (vec1[0] == vec2[0]) && (vec1[1] == vec2[1]) && (vec1[2] == vec2[2])
}
